/*
 * Copyright (c) 2016 CA.  All rights reserved.
 *
 * This software and all information contained therein is confidential
 * and proprietary and shall not be duplicated, used, disclosed or
 * disseminated in any way except as authorized by the applicable
 * license agreement, without the express written permission of CA.
 * All authorized reproductions must be marked with this language.
 *
 * EXCEPT AS SET FORTH IN THE APPLICABLE LICENSE AGREEMENT, TO THE
 * EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS SOFTWARE
 * WITHOUT WARRANTY OF ANY KIND, INCLUDING WITHOUT LIMITATION, ANY
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT WILL CA BE LIABLE TO THE END USER OR ANY
 * THIRD PARTY FOR ANY LOSS OR DAMAGE, DIRECT OR INDIRECT, FROM THE
 * USE OF THIS SOFTWARE, INCLUDING WITHOUT LIMITATION, LOST PROFITS,
 * BUSINESS INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS
 * EXPRESSLY ADVISED OF SUCH LOSS OR DAMAGE.
 *
 */

package sample.restclient;

import static sample.restclient.JSON.gson;
import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * DataTypeAdapter needed to convert arbitrary Java Objects into JSON
 *
 */
public class ObjectDataTypeAdapter extends TypeAdapter<ObjectData> {

  @Override
  public void write(final JsonWriter out, final ObjectData data) throws IOException {
       gson.toJson(data._data, Map.class, out);
  }

  @Override
  public ObjectData read(final JsonReader in) throws IOException {
    ObjectData obj = new ObjectData();
    obj._data = gson.fromJson(in, Map.class);
    for (Map.Entry<String, Object> entry : obj._data.entrySet()) {
            Object val = entry.getValue();
            if (val instanceof Iterable) {
                Set<Object> list = new HashSet<Object>();

                Iterable itb = (Iterable) val;
                Iterator iter = itb.iterator();
                while (iter.hasNext()) {
                    list.add(convertAttribute(iter.next()));
                }
                entry.setValue(list);
            } else {
                entry.setValue(convertAttribute(val));
            }
        }
        return obj;
    }

    @SuppressWarnings("unchecked")
    private static Object convertAttribute(Object input) {
        if (!(input instanceof Map)) {
            return input;
        }

        Map<String, String> map = (Map<String, String>) input;

        String xpsclass = map.get("xpsclass");
        if (xpsclass == null) {
            ObjectLink link = new ObjectLink(map.get("href"));
            link.path = map.get("path");
            link.desc = map.get("desc");
            link.id = map.get("id");
            return link;
        }

        // quick and dirty: reserialize and reparse
        String data = gson.toJson(map);
        return SmObject.fromJson(data);
    }
}
